local addon, shared = ...

local string_format = string.format

_SIGHTSEER = { UI={} }
local SS = _SIGHTSEER

local default_settings = {
	mmx=200, mmy=200, locked = false, zone={}
	}

local _sights = {
	["A563D15783DBB2F8B"] = {name="Omen Sight"},
	["A60203E8E82FC1D26"] = {name="Quantum Sight"}
}

local SS_ABILITY = false
local SS_BUFFID = false
local SS_NAME = ""

local thisUpdate = 0
local lastUpdate = 0
local ss_expire = 0
local curZone = 0

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

-- /script _SIGHTSEER.UI.mm:SetTexture("SightSeer", "img/mm_unlocked.png")
-- /script _SIGHTSEER.UI.mm:SetTexture("SightSeer", "img/mm_active.png")
-- /script _SIGHTSEER.UI.mm:SetTexture("SightSeer", "img/mm_inactive.png")

function SS.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if SightSeer_Settings == nil then
			local scx = math.floor(UIParent:GetWidth()/2)-12
			local scy = math.floor(UIParent:GetHeight()/2)-72
			SightSeer_Settings = {mmx=scx, mmy=scy}
		end
		MergeTable(SightSeer_Settings, default_settings)
		if MINIMAPDOCKER then
			MINIMAPDOCKER.Register(addon.identifier, SS.UI.mm)
		else
			SS.UI.mm:SetPoint("TOPLEFT", UIParent, "TOPLEFT", SightSeer_Settings.mmx, SightSeer_Settings.mmy)
		end
		SS.UpdateMMIcon()
		LibVersionCheck.register(addon.toc.Identifier, addon.toc.Version)
	end
end

function SS.Event_Buff_Remove(h,u,t)
	local uid = ""
	if t[SS_BUFFID] then
		SS_BUFFID = false
		lastUpdate = 0
	end
end
	
function SS.Event_Buff_Add(h,u,t)
	if u == playerID then
		for k,v in pairs(Inspect.Buff.Detail(u,t)) do
			if v.abilityNew == SS_ABILITY then
				ss_expire = v.begin + v.duration
				SS_BUFFID = k
				lastUpdate = 0
				SS.UI.warning:SetAlpha(0)
			end
		end
	end
end

function SS.Event_System_Update_Begin(h)
	thisUpdate = Inspect.Time.Frame()
	if thisUpdate - lastUpdate >= 1 then
		local td = ss_expire - thisUpdate
		if td <= 59 and td > 0 then
			SS.UI.warning:SetText(string.format("%s [0:%02ds]", SS_NAME, td))
			SS.UI.warning:SetAlpha(0.5+((60-td)/120))
		elseif SS_BUFFID == false then
			SS.UI.warning:SetText(SS_NAME.." Not Active")
			SS.UI.warning:SetAlpha(1)
		end
		lastUpdate = thisUpdate
	end
end

function SS.SetupUI()
	SS.UI.context = UI.CreateContext(addon.identifier)
	
	SS.UI.warning = UI.CreateFrame("Text", "SS.UI.warning", SS.UI.context)
	SS.UI.warning:SetVisible(false)
	SS.UI.warning:SetPoint("CENTER", UIParent, "CENTER", 0,0)
	SS.UI.warning:SetFontSize(36)
	SS.UI.warning:SetFontColor(1,0,0)
	SS.UI.warning:SetText("")
	SS.UI.warning:SetEffectGlow({strength=48, blurX=24, blurY=24})
	
	SS.UI.mm = UI.CreateFrame("Texture", "SS.UI.mm", SS.UI.context)
	SS.UI.mm:SetWidth(36)
	SS.UI.mm:SetHeight(36)

	SS.UI.mm:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if SightSeer_Settings.locked or MINIMAPDOCKER then
			if SightSeer_Settings.zone[curZone] then
				SightSeer_Settings.zone[curZone] = nil
				SS.UI.warning:SetVisible(true)
			else
				SightSeer_Settings.zone[curZone] = true
				SS.UI.warning:SetVisible(false)
			end
			SS.UpdateMMIcon()
		end
	end, "Event.UI.Input.Mouse.Left.Click")

	if MINIMAPDOCKER == nil then
		SS.UI.mm:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
			if SightSeer_Settings.locked == false then
				self.MouseDown = true
				local mouseData = Inspect.Mouse()
				self.sx = mouseData.x - SS.UI.mm:GetLeft()
				self.sy = mouseData.y - SS.UI.mm:GetTop()
			end
		end, "Event.UI.Input.Mouse.Left.Down")

		SS.UI.mm:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
			self.MouseDown = false
			if SightSeer_Settings.locked == false then
				SightSeer_Settings.mmx = SS.UI.mm:GetLeft()
				SightSeer_Settings.mmy = SS.UI.mm:GetTop()
			end
		end, "Event.UI.Input.Mouse.Left.Up")

		SS.UI.mm:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
			if SightSeer_Settings.locked == false and self.MouseDown then
				local nx, ny
				local mouseData = Inspect.Mouse()
				nx = mouseData.x - self.sx
				ny = mouseData.y - self.sy
				SS.UI.mm:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
			end
		end, "Event.UI.Input.Mouse.Cursor.Move")
		
		SS.UI.mm:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			self.MouseDown = false
			SightSeer_Settings.locked = not SightSeer_Settings.locked
			SS.UpdateMMIcon()
		end, "Event.UI.Input.Mouse.Right.Click")
	end
	
end

function SS.UpdateMMIcon()
	if MINIMAPDOCKER or SightSeer_Settings.locked then
		if SightSeer_Settings.zone[curZone] then
			SS.UI.mm:SetTexture("SightSeer", "img/mm_inactive.png")
		else
			SS.UI.mm:SetTexture("SightSeer", "img/mm_active.png")
		end
	else
		SS.UI.mm:SetTexture(addon.identifier, "img/mm_unlocked.png")
	end
end

function SS.Event_libZoneChange_Player(h, zn, zid)
	curZone = zid
	SS.UpdateMMIcon()
	if SightSeer_Settings.zone[zid] then
		SS.UI.warning:SetVisible(false)
	else
		SS.UI.warning:SetVisible(true)
	end
end

function SS.Event_Ability_New_Add(h,t)
	for k,v in pairs(t) do
		if _sights[k] then
			SS_ABILITY = k
			local d = Inspect.Ability.New.Detail(k)
			SS_NAME = d.name
			for a,b in pairs(Inspect.Buff.Detail("player", Inspect.Buff.List("player"))) do
				if b.abilityNew == SS_ABILITY then
					SS_BUFFID = a
					ss_expire = b.begin + b.duration
				end
			end
			Command.Event.Detach(Event.Ability.New.Add, nil, nil, nil, addon.identifier)
			Command.Event.Attach(Event.Buff.Add, SS.Event_Buff_Add, "Event.Buff.Add")
			Command.Event.Attach(Event.Buff.Remove, SS.Event_Buff_Remove, "Event.Buff.Remove")
			Command.Event.Attach(Event.System.Update.Begin, SS.Event_System_Update_Begin, "Event.System.Update.Begin")
			Command.Event.Attach(Library.libZoneChange.Player, SS.Event_libZoneChange_Player, "Library.LibZoneChange.Player")
		end
	end
end

SS.SetupUI()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, SS.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.Ability.New.Add, SS.Event_Ability_New_Add, "Event.Ability.New.Add")

playerID = Inspect.Unit.Lookup("player")

print(string.format("v%s loaded.", addon.toc.Version))